@@ -11,6 +11,7 @@ use solana_sdk::{
     program_utils::limited_deserialize,
     pubkey::Pubkey,
 };
+use std::collections::BTreeSet;
 
 pub fn process_instruction(
     _program_id: &Pubkey,
@@ -29,7 +30,7 @@ pub fn process_instruction(
             return Err(InstructionError::InvalidAccountOwner);
         }
 
-        deserialize(&config_account.data()).map_err(|err| {
+        deserialize(config_account.data()).map_err(|err| {
             ic_msg!(
                 invoke_context,
                 "Unable to deserialize config account: {}",
@@ -101,6 +102,15 @@ pub fn process_instruction(
         }
     }
 
+    if invoke_context.is_feature_active(&feature_set::dedupe_config_program_signers::id()) {
+        let total_new_keys = key_list.keys.len();
+        let unique_new_keys = key_list.keys.into_iter().collect::<BTreeSet<_>>();
+        if unique_new_keys.len() != total_new_keys {
+            ic_msg!(invoke_context, "new config contains duplicate keys");
+            return Err(InstructionError::InvalidArgument);
+        }
+    }
+
     // Check for Config data signers not present in incoming account update
     if current_signer_keys.len() > counter {
         ic_msg!(
@@ -120,7 +130,7 @@ pub fn process_instruction(
     config_keyed_account
         .try_account_ref_mut()?
         .data_as_mut_slice()[..data.len()]
-        .copy_from_slice(&data);
+        .copy_from_slice(data);
     Ok(())
 }
 
@@ -206,7 +216,7 @@ mod tests {
         let (_, config_account) = create_config_account(keys);
         assert_eq!(
             Some(MyConfig::default()),
-            deserialize(get_config_data(&config_account.borrow().data()).unwrap()).ok()
+            deserialize(get_config_data(config_account.borrow().data()).unwrap()).ok()
         );
     }
 
@@ -231,7 +241,7 @@ mod tests {
         );
         assert_eq!(
             Some(my_config),
-            deserialize(get_config_data(&config_account.borrow().data()).unwrap()).ok()
+            deserialize(get_config_data(config_account.borrow().data()).unwrap()).ok()
         );
     }
 
@@ -311,11 +321,11 @@ mod tests {
             ),
             Ok(())
         );
-        let meta_data: ConfigKeys = deserialize(&config_account.borrow().data()).unwrap();
+        let meta_data: ConfigKeys = deserialize(config_account.borrow().data()).unwrap();
         assert_eq!(meta_data.keys, keys);
         assert_eq!(
             Some(my_config),
-            deserialize(get_config_data(&config_account.borrow().data()).unwrap()).ok()
+            deserialize(get_config_data(config_account.borrow().data()).unwrap()).ok()
         );
     }
 
@@ -444,11 +454,11 @@ mod tests {
             ),
             Ok(())
         );
-        let meta_data: ConfigKeys = deserialize(&config_account.borrow().data()).unwrap();
+        let meta_data: ConfigKeys = deserialize(config_account.borrow().data()).unwrap();
         assert_eq!(meta_data.keys, keys);
         assert_eq!(
             new_config,
-            MyConfig::deserialize(get_config_data(&config_account.borrow().data()).unwrap())
+            MyConfig::deserialize(get_config_data(config_account.borrow().data()).unwrap())
                 .unwrap()
         );
 
@@ -493,6 +503,96 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_config_initialize_contains_duplicates_fails() {
+        solana_logger::setup();
+        let config_address = Pubkey::new_unique();
+        let signer0_pubkey = Pubkey::new_unique();
+        let signer0_account = RefCell::new(AccountSharedData::default());
+        let keys = vec![
+            (config_address, false),
+            (signer0_pubkey, true),
+            (signer0_pubkey, true),
+        ];
+        let (config_keypair, config_account) = create_config_account(keys.clone());
+        let config_pubkey = config_keypair.pubkey();
+        let my_config = MyConfig::new(42);
+
+        // Attempt initialization with duplicate signer inputs
+        let instruction = config_instruction::store(&config_pubkey, true, keys, &my_config);
+        let accounts = vec![
+            (true, false, &config_pubkey, &config_account),
+            (true, false, &signer0_pubkey, &signer0_account),
+            (true, false, &signer0_pubkey, &signer0_account),
+        ];
+        let keyed_accounts = create_keyed_accounts_unified(&accounts);
+        assert_eq!(
+            process_instruction(
+                &id(),
+                &instruction.data,
+                &mut MockInvokeContext::new(keyed_accounts)
+            ),
+            Err(InstructionError::InvalidArgument),
+        );
+    }
+
+    #[test]
+    fn test_config_update_contains_duplicates_fails() {
+        solana_logger::setup();
+        let config_address = Pubkey::new_unique();
+        let signer0_pubkey = Pubkey::new_unique();
+        let signer1_pubkey = Pubkey::new_unique();
+        let signer0_account = RefCell::new(AccountSharedData::default());
+        let signer1_account = RefCell::new(AccountSharedData::default());
+        let keys = vec![
+            (config_address, false),
+            (signer0_pubkey, true),
+            (signer1_pubkey, true),
+        ];
+        let (config_keypair, config_account) = create_config_account(keys.clone());
+        let config_pubkey = config_keypair.pubkey();
+        let my_config = MyConfig::new(42);
+
+        let instruction = config_instruction::store(&config_pubkey, true, keys, &my_config);
+        let accounts = vec![
+            (true, false, &config_pubkey, &config_account),
+            (true, false, &signer0_pubkey, &signer0_account),
+            (true, false, &signer1_pubkey, &signer1_account),
+        ];
+        let keyed_accounts = create_keyed_accounts_unified(&accounts);
+        assert_eq!(
+            process_instruction(
+                &id(),
+                &instruction.data,
+                &mut MockInvokeContext::new(keyed_accounts)
+            ),
+            Ok(()),
+        );
+
+        // Attempt update with duplicate signer inputs
+        let new_config = MyConfig::new(84);
+        let dupe_keys = vec![
+            (config_address, false),
+            (signer0_pubkey, true),
+            (signer0_pubkey, true),
+        ];
+        let instruction = config_instruction::store(&config_pubkey, false, dupe_keys, &new_config);
+        let accounts = vec![
+            (false, false, &config_pubkey, &config_account),
+            (true, false, &signer0_pubkey, &signer0_account),
+            (true, false, &signer0_pubkey, &signer0_account),
+        ];
+        let keyed_accounts = create_keyed_accounts_unified(&accounts);
+        assert_eq!(
+            process_instruction(
+                &id(),
+                &instruction.data,
+                &mut MockInvokeContext::new(keyed_accounts)
+            ),
+            Err(InstructionError::InvalidArgument),
+        );
+    }
+
     #[test]
     fn test_config_updates_requiring_config() {
         solana_logger::setup();
@@ -546,11 +646,11 @@ mod tests {
             ),
             Ok(())
         );
-        let meta_data: ConfigKeys = deserialize(&config_account.borrow().data()).unwrap();
+        let meta_data: ConfigKeys = deserialize(config_account.borrow().data()).unwrap();
         assert_eq!(meta_data.keys, keys);
         assert_eq!(
             new_config,
-            MyConfig::deserialize(get_config_data(&config_account.borrow().data()).unwrap())
+            MyConfig::deserialize(get_config_data(config_account.borrow().data()).unwrap())
                 .unwrap()
         );
 
